/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemExitHook.java 1:04:18 PM ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.system.launch;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.kernel.server.FractalHelper;
import org.objectweb.petals.kernel.server.PetalsListener;
import org.objectweb.petals.kernel.server.PetalsServer;

/**
 * This class is used to start and manage the Petals platform from command line
 *
 * @author alouis, ddesjardins, Rafael Marins, rnaudin
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class StandaloneLauncher implements PetalsListener {

    public static final String START_COMMAND = "start";

    public static final String STOP_COMMAND = "stop";

    public static final String SHUTDOWN_COMMAND = "shutdown";

    public static final String VERSION_COMMAND = "version";

    public static final String JNDIBROWSE_COMMAND = "JNDIBrowser";

    public StandaloneLauncher() {
        super();
    }

    /**
     * The launcher instance to process the main program
     */
    private static StandaloneLauncher launcher;

    /**
     * The Petals server instance
     */
    private static PetalsServer petalsServer;

    /**
     * System exit hook used if the command line is used
     */
    private static SystemExitHook systemExitHook;

    /**
     * @param args
     */
    public static void main(String[] args) {

        // Process the Command and exits abnormaly if necessary

        String command = null;
        try {
            launcher = new StandaloneLauncher();

            petalsServer = new PetalsServer();

            command = (args.length == 0 || args[0].startsWith("-")) ? START_COMMAND
                    : args[0];

            if (STOP_COMMAND.equals(command)) {
                System.out.println("Petals is stopping...");
                petalsServer.triggerStop(false);
                System.out.println("Petals is stopped");
            } else if (SHUTDOWN_COMMAND.equals(command)) {
                System.out.println("Petals is stopping...");
                petalsServer.triggerStop(true);
                System.out.println("Petals is stopped");
            } else if (VERSION_COMMAND.equals(command)) {
                petalsServer.getServerVersion();
            } else if (JNDIBROWSE_COMMAND.equals(command)) {
                System.out.println(petalsServer.browseJNDI());
            } else if (START_COMMAND.equals(command)) {

                // show the fractal explorer mode if asked
                if (args.length > 0 && "-explore".equalsIgnoreCase(args[0])) {
                    petalsServer.setObserver(true);
                }

                // check if started or not stopped correctly
                if (petalsServer.isLocked()) {
                    throw new PetalsException(
                            "Can not start the PEtALS server, remove lock file or stop server");
                }

                petalsServer.addStopListener(launcher);

                // add a hook if a terminate signal is sent from the command
                // line
                systemExitHook = new SystemExitHook(petalsServer);
                Runtime.getRuntime().addShutdownHook(systemExitHook);

                System.out.println("Petals is starting...");
                petalsServer.init(false);
                petalsServer.start();
                System.out
                        .println("##### Petals container correctly started #####");

                // show the commandLine mode if asked
                if (args.length > 0 && "-console".equalsIgnoreCase(args[0])) {
                    launcher.commandLineMode();
                }
            }
        } catch (Throwable e) {
            System.err.println("Command processing error: " + command);
            e.printStackTrace();
            System.exit(-1);
        }
    }

    /**
     * Show a command line to interact with Petals TODO we pass the PetalsAdmin
     * object to the CommandReader to perform some actions. Find a better way.
     *
     * @throws NoSuchInterfaceException
     * @throws ADLException
     * @throws PetalsException
     */
    protected void commandLineMode() throws NoSuchInterfaceException,
            ADLException, PetalsException {
        Component systemState = FractalHelper.getFractalHelper()
                .getComponentByName(petalsServer.getPetalsContentController(),
                        FractalHelper.PETALSADMIN_COMPONENT);

        PetalsAdminServiceMBean petalsService = (PetalsAdminServiceMBean) systemState
                .getFcInterface("service");

        CommandReader console = new CommandReader(petalsService, petalsServer);

        console.read();
    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStarted()
     */
    public void onPetalsStarted() {

    }

    /*
     * (non-Javadoc)
     * @see org.objectweb.petals.kernel.server.PetalsListener#onPetalsStopped()
     */
    public void onPetalsStopped() {
        // remove the System Hook to avoid the Petals server stop() called a
        // second time
        Runtime.getRuntime().removeShutdownHook(systemExitHook);
        System.out.println("Petals is stopped");

        System.exit(0);
    }
}
